package dev.flyfish.framework.domain.authorized;

import dev.flyfish.framework.domain.base.NameLikeQo;
import dev.flyfish.framework.domain.po.Department;
import dev.flyfish.framework.query.LambdaQueryChain;
import dev.flyfish.framework.query.Queries;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;

import java.util.Collections;
import java.util.Set;

/**
 * 公共方法封装
 *
 * @param <T> 泛型
 */
@Setter
public abstract class AbstractAuthorizedQo<T extends AuthorizedDomain> extends NameLikeQo<T> {

    // 是否已发布
    protected Boolean published;

    /**
     * 获取可见的权限ids
     *
     * @return 结果
     */
    protected Set<String> authorizeIds() {
        if (user instanceof AuthorizedUserDetails) {
            return ((AuthorizedUserDetails) user).getAuthorityCodes();
        }
        return Collections.singleton(Department.PUBLIC);
    }

    /**
     * 构造发布状态查询
     *
     * @return 结果
     */
    protected LambdaQueryChain<T> withPublished() {
        if (null != published) {
            if (BooleanUtils.isTrue(published)) {
                return Queries.where(T::getPublished).eq(true);
            }
            return Queries.within(
                    Queries.where(T::getPublished).isNull().or(T::getPublished).eq(false)
            );
        } else {
            // 未指定发布状态，查询已发布内容
            return Queries.where(T::getPublished).eq(true);
        }
    }

}
