package dev.flyfish.framework.mongodb.operations;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.query.Query;
import dev.flyfish.framework.repository.ReactiveEntityOperations;
import lombok.RequiredArgsConstructor;
import org.springframework.data.mongodb.core.ReactiveMongoOperations;
import org.springframework.data.mongodb.repository.query.MongoEntityInformation;
import org.springframework.data.repository.core.EntityInformation;
import org.springframework.data.util.CastUtils;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RequiredArgsConstructor
public class MongoReactiveEntityOperations implements ReactiveEntityOperations {

    private final ReactiveMongoOperations reactiveOperations;

    /**
     * 查询，通过查询条件和类进行精确查询
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @return 查询结果
     */
    @Override
    public <T extends Domain, E extends EntityInformation<T, String>> Mono<T> find(Query query, E entityInformation) {
        return reactiveOperations.findOne(toNative(query, entityInformation), cast(entityInformation).getJavaType());
    }

    /**
     * 查询全部符合条件的实体
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @return 结果
     */
    @Override
    public <T extends Domain, E extends EntityInformation<T, String>> Flux<T> findAll(Query query, E entityInformation) {
        return reactiveOperations.find(toNative(query, entityInformation), cast(entityInformation).getJavaType());
    }

    private <T extends Domain> MongoEntityInformation<T, String> cast(EntityInformation<T, String> entityInformation) {
        return CastUtils.cast(entityInformation);
    }

    /**
     * 保存实体
     *
     * @param bean 实体
     * @return 结果
     */
    @Override
    public <T extends Domain> Mono<T> save(T bean) {
        return reactiveOperations.save(bean);
    }
}
