package dev.flyfish.framework.r2dbc.metadata.visitor;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.r2dbc.metadata.reference.R2dbcAssociation;
import dev.flyfish.framework.r2dbc.metadata.reference.R2dbcCollection;
import dev.flyfish.framework.r2dbc.metadata.reference.R2dbcRelation;
import org.springframework.data.mapping.PersistentPropertyAccessor;
import org.springframework.lang.Nullable;
import reactor.core.publisher.Mono;

/**
 * 元数据处理器
 *
 * @author wangyu
 */
public interface MetadataHandler {

    /**
     * 处理association
     *
     * @param association 一对一关联
     * @param accessor    对象访问器
     * @return 结果
     */
    @Nullable
    Mono<Domain> handleAssociation(R2dbcAssociation association, PersistentPropertyAccessor<Domain> accessor);

    /**
     * 处理collection
     *
     * @param collection 一对多关联
     * @param accessor   对象访问器
     * @return 结果
     */
    @Nullable
    Mono<Domain> handleCollection(R2dbcCollection collection, PersistentPropertyAccessor<Domain> accessor);

    /**
     * 处理relation
     *
     * @param relation 多对多关联
     * @param accessor 访问器
     * @return 结果
     */
    @Nullable
    Mono<Domain> handleRelation(R2dbcRelation relation, PersistentPropertyAccessor<Domain> accessor);
}
