package dev.flyfish.framework.r2dbc;

import dev.flyfish.framework.r2dbc.config.R2dbcDataConfig;
import dev.flyfish.framework.r2dbc.domain.TestDO;
import dev.flyfish.framework.r2dbc.domain.TestQO;
import dev.flyfish.framework.r2dbc.repository.TestAssoRepository;
import dev.flyfish.framework.r2dbc.repository.TestRepository;
import dev.flyfish.framework.utils.JacksonUtil;
import group.flyfish.fluent.chain.SQL;
import group.flyfish.fluent.operations.R2dbcFluentSQLOperations;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.r2dbc.core.DatabaseClient;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.List;

@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = R2DbcRepositoryTest.class)
@SpringBootApplication(scanBasePackages = "dev.flyfish.framework")
@Import(R2dbcDataConfig.class)
@Slf4j
public class R2DbcRepositoryTest {

    @Resource
    private TestRepository testRepository;
    @Autowired
    private TestAssoRepository testAssoRepository;
    @Resource
    private DatabaseClient databaseClient;

    @Test
    public void test() {
        TestQO qo = new TestQO();
        qo.setName("1");
        // 开启关联查询
        qo.setFetchRef(true);

        TestDO test = new TestDO();
        test.setId("100");
        test.setCode("ttt");
        test.setName("测试名称");
        test.setOtherId("1");

        // 打印插入结果
        testRepository.insert(test)
                .doOnNext(obj -> this.log("插入结果:{}", obj))
                .block();
        // 查询生成
        testRepository.findByNameContaining("名")
                .doOnNext(obj -> this.log("生成查询:{}", obj))
                .block();
        List<TestDO> all = testRepository.findAll(qo).collectList()
                .doOnNext(obj -> this.log("qo查询：{}", obj))
                .block();
        // 修改值并保存测试
        all.get(0).getAsso().setName("修改后的值");
        testRepository.saveAll(all).collectList()
                .doOnNext(obj -> this.log("批量保存：{}", obj))
                .block();
        testAssoRepository.findById("1")
                .doOnNext(obj -> this.log("id查询：{}", obj))
                .block();
        // 测试fluent sql
        SQL.bind(new R2dbcFluentSQLOperations(databaseClient));
        testRepository.getDTOById(100L)
                .doOnNext(obj -> this.log("测试fluent:{}", obj))
                .block();
    }

    private void log(String msg, Object bean) {
        log.info(msg, JacksonUtil.toPrettyJson(bean));
    }
}
