package dev.flyfish.framework.file.controller;

import dev.flyfish.framework.bean.Result;
import dev.flyfish.framework.beans.annotations.RestMapping;
import dev.flyfish.framework.file.config.UploadConfiguration;
import dev.flyfish.framework.file.domain.Attachment;
import dev.flyfish.framework.file.service.AttachmentService;
import dev.flyfish.framework.file.utils.DownloadUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.codec.multipart.FilePart;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestPart;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 附件上传相关
 *
 * @author wangyu
 */
@RestMapping("/attachment")
public class AttachmentUploadController {

    @Resource
    private UploadConfiguration configuration;
    @Resource
    private AttachmentService attachmentService;

    /**
     * 上传媒体，支持多个同时上传
     *
     * @param files 文件
     * @return 结果
     */
    @PostMapping("")
    public Mono<Result<List<Attachment>>> uploadAttachment(@RequestPart("file") Flux<FilePart> files) {
        return files.flatMap(attachmentService::upload)
                .reduce(Result.accept(new ArrayList<>()), ((listResult, attachment) -> {
                    listResult.data().add(attachment);
                    return listResult;
                }));
    }

    @GetMapping("/**")
    public Mono<Void> downloadStatic(ServerHttpRequest request, ServerHttpResponse response) {
        String path = StringUtils.substringAfterLast(request.getURI().getPath(), "/attachment");
        return DownloadUtils.download(configuration.getLocalPath() + path, response);
    }

    @GetMapping("/downloads/{id}")
    public Mono<Void> downloadAttachment(@PathVariable String id, ServerHttpResponse response) {
        return attachmentService.getById(id)
                .flatMap(attachment -> DownloadUtils.download(configuration.getLocalPath() +
                        attachment.getPath(), response));
    }
}
