package dev.flyfish.framework.user.service;

import dev.flyfish.framework.user.repository.UserRepository;
import dev.flyfish.framework.user.domain.UserQo;
import dev.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import dev.flyfish.framework.domain.base.Qo;
import dev.flyfish.framework.domain.po.User;
import dev.flyfish.framework.enums.UserType;
import dev.flyfish.framework.service.UserFindService;
import dev.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Set;

/**
 * 异步用户service
 *
 * @author wangyu
 */
@Service
public class UserService extends BaseReactiveServiceImpl<User> implements UserFindService {

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    @Override
    public Mono<User> findByUsername(String username) {
        return ((UserRepository) repository).findByUsername(username);
    }

    /**
     * 分页查询，这里使用作用域查询
     *
     * @param query 查询实体
     * @return 结果
     */
    @Override
    public Mono<Page<User>> getPageList(Qo<User> query) {
        // 非超级管理员，永远返回作用域的部门人员
        if (query.getUser().getType() != UserType.SUPER_ADMIN) {
            AuthorizedUserDetails userDetails = (AuthorizedUserDetails) query.getUser();
            UserQo qo = (UserQo) query;
            if (CollectionUtils.isEmpty(qo.getDepartments())) {
                Set<String> visibleDeparts = userDetails.getVisibleDeparts();
                if (CollectionUtils.isNotEmpty(visibleDeparts)) {
                    qo.setDepartments(new ArrayList<>(visibleDeparts));
                } else {
                    qo.setDepartments(Collections.emptyList());
                }
            }
        }
        return super.getPageList(query);
    }
}
